@@ -56,7 +56,7 @@ use Carp qw/croak carp confess cluck/;
 use Scalar::Util qw/looks_like_number blessed reftype/;
 use Unicode::UTF8 qw/decode_utf8 valid_utf8/;
 use B;
-our $VERSION = '0.23';
+our $VERSION = '0.26_03';
 
 # http://stackoverflow.com/questions/1185822/how-do-i-create-or-test-for-nan-or-infinity-in-perl#1185828
 
@@ -287,6 +287,11 @@ sub closeB
 	$jc->newline_indent ();
     }
     $jc->{output} .= $b;
+    if ($jc->{_indent}) {
+	if ($jc->{depth} == 0) {
+	    $jc->{output} .= "\n";
+	}
+    }
 }
 
 sub comma
